#include <iostream>
#include <vector>
#include <string>
#include <algorithm>


using namespace std;


#define int long long


const int MAXN = 2001;
const int ALP = 94;
const int MAXS = ALP * ALP + 1;


int a[MAXN][MAXN];
int hhash[MAXN][MAXN];
int vhash[MAXN][MAXN];


const int M = (int)1e9 + 7;
const int P = 179;

int PPOW[MAXS];


int mod(int x) {
    x %= M;

    if (x < 0) {
        return x + M;
    }

    return x;
}


int hh(int i, int j, int len) {
    return mod(hhash[i][j + len] - hhash[i][j]);
}

int vh(int i, int j, int len) {
    return mod(vhash[i + len][j] - vhash[i][j]);
}


#define left(v) 2 * v + 1
#define right(v) 2 * v + 2
#define parent(v) (v - 1) / 2

struct LST {
    vector<signed> data;
    int m;

    void update(int v) {
        if (data[left(v)] == data[right(v)]) {
            data[v] = data[left(v)];
        } else {
            data[v] = -1;
        }
    }

    LST() {};

    LST(vector<int> a) {
        int n = a.size();

        for (m = 1; m < n; m <<= 1);

        data.resize(2 * m + 1);

        for (int i = 0; i < n; ++i) {
            data[m + i] = a[i];
        }

        for (int i = m - 1; i >= 0; --i) {
            update(i);
        }
    }

    int get(int L, int R) {
        L += m;
        R += m - 1;

        int res = 0;
        while (L < R) {
            if (L % 2 == 0) {
                if (res == 0) {
                    res = data[L];
                }
                if (res != data[L]) {
                    res = -1;
                }

                ++L;
            }

            L = parent(L);

            if (R % 2 == 1) {
                if (res == 0) {
                    res = data[R];
                }
                if (res != data[R]) {
                    res = -1;
                }

                --R;
            }

            R = parent(R);
        }

        if (L == R) {
            if (res == 0) {
                res = data[L];
            }
            if (res != data[L]) {
                res = -1;
            }
        }

        return res;
    }
};


signed main() {
    PPOW[0] = 1;
    for (int i = 1; i < MAXS; ++i) {
        PPOW[i] = PPOW[i - 1] * P % M;
    }

    ios_base::sync_with_stdio(false);
    cin.tie(0);
    cout.tie(0);

    int n, m;
    cin >> n >> m;

    char c1, c2;
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            cin >> c1 >> c2;
            
            c1 -= 33;
            c2 -= 33;

            a[i][j] = 1 + c1 * ALP + c2;
        }
    }

    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            int cur = a[i][j];

            hhash[i][j + 1] = (hhash[i][j] + cur * PPOW[cur]) % M;
            vhash[i + 1][j] = (vhash[i][j] + cur * PPOW[cur]) % M;
        }
    }

    vector<vector<LST>> ht(m, vector<LST>(m + 1));
    vector<vector<LST>> vt(n, vector<LST>(n + 1));
    for (int i = 0; i < m; ++i) {
        for (int j = i + 1; j <= m; ++j) {
            vector<int> res(n);
            for (int k = 0; k < n; ++k) {
                res[k] = hh(k, i, j - i);
            }

            ht[i][j] = LST(res);
        }
    }

    for (int i = 0; i < n; ++i) {
        for (int j = i + 1; j <= n; ++j) {
            vector<int> res(m);
            for (int k = 0; k < m; ++k) {
                res[k] = vh(i, k, j - i);
            }

            vt[i][j] = LST(res);
        }
    }

    int answ = 0;
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            vector<int> used(MAXS);

            for (int k = 0; k < min(n - i, m - j); ++k) {
                int cur = a[i][j + k];

                if (used[cur]) {
                    break;
                }

                used[cur] = 1;

                int ch = hh(i, j, k + 1);

                bool b = (ht[j][j + k + 1].get(i, i + k + 1) == ch);
                b = b && (vt[i][i + k + 1].get(j, j + k + 1) == ch);

                answ += b;
            }
        }
    }

    cout << answ;

    return 0;
}
