#include <iostream>
#include <vector>
#include <map>
#include <algorithm>
#include <string>
#include <ctime>

using namespace std;

const int ALP = 127 - 33;
const long long Q = 2286661337;

int cnt[ALP * ALP];
vector <long long> prime, r_prime;
vector <vector <long long>> hor, r_hor, ver, r_ver;

long long bin_pow(long long x, long long p) {
	if (p == 0)
		return 1;
	long long tmp = bin_pow(x, p / 2);
	if (p % 2)
		return tmp * tmp % Q * x % Q;
	else
		return tmp * tmp % Q;
}

void build(vector <string>& a, int n, int m) {
	for (auto& u : a)
		for (auto& w : u)
			w -= 33;
	hor.resize(n, vector <long long>(m));
	ver = r_ver = r_hor = hor;
	for (int i = 0; i < n; i++) {
		hor[i][0] = prime[a[i][0] * ALP + a[i][1]];
		r_hor[i][0] = r_prime[a[i][0] * ALP + a[i][1]];
	}
	for (int i = 0; i < n; i++) {
		for (int j = 1; j < m; j++) {
			hor[i][j] = hor[i][j - 1] * prime[a[i][2 * j] * ALP + a[i][2 * j + 1]] % Q;
			r_hor[i][j] = r_hor[i][j - 1] * r_prime[a[i][2 * j] * ALP + a[i][2 * j + 1]] % Q;
		}
	}
	for (int j = 0; j < m; j++) {
		ver[0][j] = prime[a[0][2 * j] * ALP + a[0][2 * j + 1]];
		r_ver[0][j] = r_prime[a[0][2 * j] * ALP + a[0][2 * j + 1]];
	}
	for (int i = 1; i < n; i++) {
		for (int j = 0; j < m; j++) {
			ver[i][j] = ver[i - 1][j] * prime[a[i][2 * j] * ALP + a[i][2 * j + 1]] % Q;
			r_ver[i][j] = r_ver[i - 1][j] * r_prime[a[i][2 * j] * ALP + a[i][2 * j + 1]] % Q;
		}
	}
}

bool check(int x, int y, int k, vector <string> &a) {
	bool cor = 1;
	for (int i = x; i < x + k; i++) {
		if (cnt[a[i][y] * ALP + a[i][y + 1]]) cor = 0;
		else cnt[a[i][y] * ALP + a[i][y + 1]]++;
	}
	for (int i = x; i < x + k; i++) {
		cnt[a[i][y] * ALP + a[i][y + 1]] = 0;
	}
	if (!cor) return 0;
	cor = 1;
	for (int j = y; j < y + 2 * k; j += 2) {
		if (cnt[a[x][j] * ALP + a[x][j + 1]]) cor = 0;
		else cnt[a[x][j] * ALP + a[x][j + 1]]++;
	}
	for (int j = y; j < y + 2 * k; j += 2) {
		cnt[a[x][j] * ALP + a[x][j + 1]] = 0;
	}
	if (!cor) return 0;
	y /= 2;
	long long hor_h = hor[x][y + k - 1] * (y == 0 ? 1 : r_hor[x][y - 1]) % Q;
	long long ver_h = ver[x + k - 1][y] * (x == 0 ? 1 : r_ver[x - 1][y]) % Q;
	for (int i = 1; i < k; i++) {
		if (hor[x + i][y + k - 1] * (y == 0 ? 1 : r_hor[x + i][y - 1]) % Q != hor_h)
			return 0;
		if (ver[x + k - 1][y + i] * (x == 0 ? 1 : r_ver[x - 1][y + i]) % Q != ver_h)
			return 0;
	}
	return 1;
}

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
	for (int i = 17; prime.size() < ALP * ALP; i++) {
		bool tmp = 1;
		for (int j = 2; j * j <= i && tmp; j++)
			if (i % j == 0)
				tmp = 0;
		if (tmp) {
			prime.emplace_back(i);
			r_prime.emplace_back(bin_pow(i, Q - 2));
		}
		
	}
	int n, m;
	cin >> n >> m;
	vector <string> a(n);
	for (auto& u : a)
		cin >> u;
	build(a, n, m);
	int ans = 0;
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < 2 * m; j += 2) {
			for (int k = 1; j + 2 * k <= 2 * m && i + k <= n; k++)
				if (check(i, j, k, a))
					ans++;
		}
	}
	cout << ans;
}