#include<iostream>
#include<vector>
#include<algorithm>
#include<string>
using namespace std;

template<typename T>
class trie {
	vector<T> tree;
	vector<T> a;
	int n = 0;
	void _build(int v, int l, int r) {
		if (r - l == 1) {
			tree[v] = a[l];
			return;
		}
		int m = (l + r) / 2;
		_build(2 * v, l, m);
		_build(2 * v + 1, m, r);
		tree[v] = min(tree[2 * v], tree[2 * v + 1]);
	}
	T _get(int v, int l, int r, int tl, int tr) {
		if (l <= tl && tr <= r) {
			return tree[v];
		}
		if (tr <= l || r <= tl) {
			return 1e9;
		}
		int m = (tl + tr) / 2;
		return min(_get(2 * v, l, r, tl, m), _get(2 * v + 1, l, r, m, tr));
	}
	void _update(int v, int l, int r, int pos, const T& x) {
		if (r - l == 1) {
			tree[v] = x;
			a[v] = x;
			return;
		}
		int m = (l + r) / 2;
		if (pos < m) {
			_update(2 * v, l, m, pos, x);
		}
		else {
			_update(2 * v + 1, m, r, pos, x);
		}
		tree[v] = min(tree[2 * v], tree[2 * v + 1]);
	}
	void _add(int v, int l, int r, int pos, const T& x) {
		if (r - l == 1) {
			tree[v] += x;
			a[l] += x;
			return;
		}
		int m = (l + r) / 2;
		if (pos < m) {
			_add(2 * v, l, m, pos, x);
		}
		else {
			_add(2 * v + 1, m, r, pos, x);
		}
		tree[v] = min(tree[2 * v], tree[2 * v + 1]);
	}
public:
	trie() {};
	trie(int _n) {
		n = _n;
		tree.resize(4 * n);
		a.resize(n);
	};
	void build(const vector<T>& _a) {
		a = _a;
		n = a.size();
		tree.resize(4 * n);
		_build(1, 0, n);
	};
	T get(int l, int r) {
		return _get(1, l, r + 1, 0, n);
	};
	void update(int pos, const T& x) {
		_update(1, 0, n, pos, x);
	}
	void add(int pos, const T& x) {
		_add(1, 0, n, pos, x);
	}
	T get(int i) {
		return get(i, i);
	}
};

int main() {
	int n, m;
	cin >> n >> m;
	vector<vector<pair<char, char>>> A(n, vector<pair<char, char>>(m));
	for (int i = 0; i < n; i++) {
		string str;
		getline(cin, str);
		for (int j = 0; j < m; j++) {
			A[i][j] = {str[2 * j], str[2 * j + 1]};
		}
	}
	vector<vector<vector<int>>> ver(m, vector<vector<int>>(127, vector<int>(127)));
	trie<int> ver_cnt(m);
	vector<vector<int>> ans(n, vector<int>(m));
	for (int i = 0; i < n; i++) {
		int cnt = 0;
		for (int j = 0; j < m; j++) {
			bool good = true;
			while (good == true || ver_cnt.get(j) < n) {
				int u = ver_cnt.get(j);
				
				if (!ver[j][A[u][j].first][A[u][j].second]) {
					ver[j][A[u][j].first][A[u][j].second] = 1;
					ver_cnt.add(j, 1);
				}
				else {
					good = false;
				}
			}
		}
		for (int j = 0; j < m; j++) {
			bool good = true;
			cnt = max(cnt, j);
			while (good && cnt < m) {
				if (cnt - j + 1 < ver_cnt.get(j, cnt) - i + 1) {
					cnt++;
				}
				else {
					good = false;
				}
			}
			ans[i][j] = cnt - j;
		}
		for (int j = 0; j < m; j++) {
			ver[j][A[i][j].first][A[i][j].second] = 0;
		}
	}

	vector<vector<vector<int>>> hor(n, vector<vector<int>>(127, vector<int>(127)));
	trie<int> hor_cnt(n);
	for (int j = 0; j < m; j++) {
		int cnt = 0;
		for (int i = 0; i < n; i++) {
			bool good = true;
			while (good == true || hor_cnt.get(i) < m) {
				int u = hor_cnt.get(i);

				if (!hor[i][A[u][j].first][A[u][j].second]) {
					hor[i][A[u][j].first][A[u][j].second] = 1;
					hor_cnt.add(i, 1);
				}
				else {
					good = false;
				}
			}
		}
		for (int i = 0; i < n; i++) {
			bool good = true;
			cnt = max(cnt, i);
			while (good && cnt < n) {
				if (cnt - i + 1 < hor_cnt.get(i, cnt) - j + 1) {
					cnt++;
				}
				else {
					good = false;
				}
			}
			ans[i][j] = min(ans[i][j], cnt - i);
		}
		for (int i = 0; i < n; i++) {
			hor[i][A[i][j].first][A[i][j].second] = 0;
		}
	}
	long long result = 0;
	for (auto& str : ans) {
		for (auto& e : str) {
			result += e;
		}
	}
	cout << result;
}