#include <vector>
#include <map>
#include <iostream>

#pragma GCC optimize("O3")
#pragma GCC target("sse,sse2,sse3,ssse3,popcnt,abm,mmx,avx")

const bool IS_LOCAL = false;
const bool IS_TESTING = false;

typedef int TEST_TYPE;
typedef int RESULT_TYPE;

int n, m;
std::vector<std::vector<int>> field;

const long long mod = 1500000001;
std::vector<std::vector<long long>> line_hash, column_hash;
std::vector<std::vector<int>> max_len_super;

bool is_square_2(int x, int y, int k) {
    long long b_h = (line_hash[x][y + k] - line_hash[x][y] + 5 * mod) % mod;
    for (int i = x; i < x + k; ++i) {
        if ((line_hash[i][y + k] - line_hash[i][y] + 5 * mod) % mod != b_h) return false;
    }
    for (int j = y; j < y + k; ++j) {
        if ((column_hash[j][x + k] - column_hash[j][x] + 5 * mod) % mod != b_h) return false;
    }
    return true;
}

const int FUN = 179179179;

void prepare() {
    line_hash.resize(n);
    for (int i = 0; i < n; ++i) {
        line_hash[i].emplace_back(FUN);
        for (int j = 0; j < m; ++j) {
            line_hash[i].emplace_back((line_hash[i][j] + field[i][j] * field[i][j] * field[i][j] + FUN) % mod);
        }
    }
    column_hash.resize(m);
    for (int i = 0; i < m; ++i) {
        column_hash[i].emplace_back(FUN);
        for (int j = 0; j < n; ++j) {
            column_hash[i].emplace_back((column_hash[i][j] + field[j][i] * field[j][i] * field[j][i] + FUN) % mod);
        }
    }
    max_len_super.resize(n);
    for (int i = 0; i < n; ++i) {
        int now_left = 0;
        std::map<int, int> cnt;
        for (int j = 0; j < m; ++j) {
            cnt[field[i][j]]++;

            if (cnt[field[i][j]] > 1) {
                while (field[i][now_left] != field[i][j]) {
                    cnt[field[i][now_left]]--;
                    now_left++;
                }
                cnt[field[i][now_left]]--;
                now_left++;
            }
            max_len_super[i].emplace_back(j - now_left + 1);
        }
    }
}

RESULT_TYPE stupid() {
    prepare();
    int answer = 0;
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            for (int k = 2; k <= std::min(n - i, m - j); ++k) {              //500 * 500 * 250
                if (max_len_super[i][j + k - 1] < k) break;
                answer += is_square_2(i, j, k);
            }
        }
    }
    return answer + (n * m);
}

void read() {
    std::cin.tie();
    std::cout.tie();
    std::iostream::sync_with_stdio(false);
    std::cin >> n >> m;
    field.resize(n);
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            char t;
            std::cin >> t;
            int now = t * 127;
            std::cin >> t;
            now += t;
            field[i].emplace_back(now);
        }
    }
}

int main() {
    read();
    std::cout << stupid() << '\n';
}