#include <vector>
#include <map>
#include <iostream>

#pragma GCC optimize("O3,unroll-loops")
#pragma GCC target("sse,sse2,sse3,ssse3,popcnt,abm,mmx,avx")

int n, m;
const int MAX_N = 503, MAX_M = 503;
int field[MAX_N][MAX_M];

const long long mod = 1500000001;
long long line_hash[MAX_N][MAX_M], column_hash[MAX_N][MAX_M];
long long cashed_field[MAX_N][MAX_M];
int max_len_super[MAX_N][MAX_M];

inline bool is_square_2(const int &x, const int &y, const int &k) {
    long long b_h = (line_hash[x][y + k] - line_hash[x][y] + 5 * mod) % mod;
    for (int i = x; i < x + k; ++i) {
        if ((line_hash[i][y + k] - line_hash[i][y] + 5 * mod) % mod != b_h) return false;
    }
    for (int j = y; j < y + k; ++j) {
        if ((column_hash[j][x + k] - column_hash[j][x] + 5 * mod) % mod != b_h) return false;
    }
    return true;
}

const int FUN = 179179179;

inline void prepare() {
    for (int i = 0; i < n; ++i) {
        line_hash[i][0] = FUN;
        for (int j = 0; j < m; ++j) {
            line_hash[i][j + 1] = (line_hash[i][j] + cashed_field[i][j] + FUN) % mod;
        }
    }
    for (int i = 0; i < m; ++i) {
        column_hash[i][0] = FUN;
        for (int j = 0; j < n; ++j) {
            column_hash[i][1 + j] = (column_hash[i][j] + cashed_field[j][i] + FUN) % mod;
        }
    }
    for (int i = 0; i < n; ++i) {
        int now_left = 0;
        std::map<int, int> cnt;
        for (int j = 0; j < m; ++j) {
            cnt[field[i][j]]++;

            if (cnt[field[i][j]] == 2) {
                while (field[i][now_left] != field[i][j]) {
                    cnt[field[i][now_left]]--;
                    now_left++;
                }
                cnt[field[i][now_left]]--;
                now_left++;
            }
            max_len_super[i][j] = j - now_left + 1;
        }
    }
}

inline int stupid() {
    prepare();
    int answer = 0;
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            for (int k = 2; k <= std::min(n - i, m - j); ++k) {              //500 * 500 * 250
                if (max_len_super[i][j + k - 1] < k) break;
                answer += is_square_2(i, j, k);
            }
        }
    }
    return answer + (n * m);
}

inline void read() {
    std::cin.tie();
    std::cout.tie();
    std::iostream::sync_with_stdio(false);
    std::cin >> n >> m;
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            char t;
            std::cin >> t;
            int now = t * 127;
            std::cin >> t;
            now += t;
            field[i][j] = now;
            cashed_field[i][j] = now * now * now;
        }
    }
}

int main() {
    read();
    std::cout << stupid() << '\n';
}